@@ -11,7 +11,7 @@ import           Data.CaseInsensitive  (foldCase, foldedCase, mk)
 import           Data.Either           (isRight)
 import           Data.List.Split       (wordsBy)
 import           Data.Maybe            (fromMaybe, isJust)
-import           Data.Monoid           ((<>))
+import           Data.Semigroup        (Semigroup (..))
 import qualified Data.Text             as T
 import           Data.Time             (UTCTime, defaultTimeLocale, parseTimeM,
                                         rfc822DateFormat)
@@ -84,15 +84,13 @@ notLongerThan maxAllowed
 -- /Since 0.0.0.0/
 onlyJsonObjects :: ResponsePredicate
 onlyJsonObjects
-  = ResponsePredicate (\resp -> case go resp of
-      Nothing -> throw $ PredicateFailure "onlyJsonObjects" Nothing resp
-      Just () -> return ())
-    where
-      go r = do
-        ctyp <- lookup "content-type" (first foldedCase <$> responseHeaders r)
-        when ("application/json" `SBS.isPrefixOf` ctyp) $ do
-          (_ :: Object) <- decode (responseBody r)
-          return ()
+  = ResponsePredicate (\resp -> do
+    case lookup "content-type" (first foldedCase <$> responseHeaders resp) of
+      Nothing    -> return ()
+      Just ctype -> when ("application/json" `SBS.isPrefixOf` ctype) $ do
+        case (decode (responseBody resp) :: Maybe Object) of
+          Nothing -> throw $ PredicateFailure "onlyJsonObjects" Nothing resp
+          Just _  -> return ())
 
 -- | __Optional__
 --
@@ -189,14 +187,15 @@ getsHaveLastModifiedHeader
 notAllowedContainsAllowHeader :: RequestPredicate
 notAllowedContainsAllowHeader
   = RequestPredicate $ \req mgr -> do
-      resp <- mapM (flip httpLbs mgr) $ [ req { method = renderStdMethod m }
-                                        | m <- [minBound .. maxBound ]
-                                        , renderStdMethod m /= method req ]
-      case filter pred' resp of
-        (x:_) -> throw $ PredicateFailure "notAllowedContainsAllowHeader" (Just req) x
+      let reqs = [ req { method = renderStdMethod m } | m <- [minBound .. maxBound]
+                                                      , renderStdMethod m /= method req ]
+      resp <- mapM (flip httpLbs mgr) reqs
+
+      case filter pred' (zip reqs resp) of
+        (x:_) -> throw $ PredicateFailure "notAllowedContainsAllowHeader" (Just $ fst x) (snd x)
         []     -> return resp
     where
-      pred' resp = responseStatus resp == status405 && not (hasValidHeader "Allow" go resp)
+      pred' (_, resp) = responseStatus resp == status405 && not (hasValidHeader "Allow" go resp)
         where
           go x = all (\y -> isRight $ parseMethod $ SBSC.pack y)
                $ wordsBy (`elem` (", " :: [Char])) (SBSC.unpack x)
@@ -377,9 +376,12 @@ newtype ResponsePredicate = ResponsePredicate
   { getResponsePredicate :: Response LBS.ByteString -> IO ()
   } deriving (Generic)
 
+instance Semigroup ResponsePredicate where
+  ResponsePredicate a <> ResponsePredicate b = ResponsePredicate $ \x -> a x >> b x
+
 instance Monoid ResponsePredicate where
   mempty = ResponsePredicate $ const $ return ()
-  ResponsePredicate a `mappend` ResponsePredicate b = ResponsePredicate $ \x -> a x >> b x
+  mappend = (<>)
 
 -- | A predicate that depends on both the request and the response.
 --
@@ -391,7 +393,11 @@ newtype RequestPredicate = RequestPredicate
 -- TODO: This isn't actually a monoid
 instance Monoid RequestPredicate where
   mempty = RequestPredicate (\r m -> httpLbs r m >>= \x -> return ([x]))
-  RequestPredicate a `mappend` RequestPredicate b = RequestPredicate $ \r mgr ->
+  mappend = (<>)
+
+-- TODO: This isn't actually a monoid
+instance Semigroup RequestPredicate where
+  RequestPredicate a <> RequestPredicate b = RequestPredicate $ \r mgr ->
     liftM2 (<>) (a r mgr) (b r mgr)
 
 -- | A set of predicates. Construct one with 'mempty' and '<%>'.
@@ -400,10 +406,13 @@ data Predicates = Predicates
   , responsePredicates :: ResponsePredicate
   } deriving (Generic)
 
+instance Semigroup Predicates where
+  a <> b = Predicates (requestPredicates a <> requestPredicates b)
+                      (responsePredicates a <> responsePredicates b)
+
 instance Monoid Predicates where
   mempty = Predicates mempty mempty
-  a `mappend` b = Predicates (requestPredicates a <> requestPredicates b)
-                             (responsePredicates a <> responsePredicates b)
+  mappend = (<>)
 
 class JoinPreds a where
   joinPreds :: a -> Predicates -> Predicates
